@@ -12,7 +12,7 @@ import (
 	"net/http"
 	"os"
 	"path/filepath"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -46,26 +46,28 @@ func MountExists(specMounts []spec.Mount, dest string) bool {
 	return false
 }
 
-type byDestination []spec.Mount
-
-func (m byDestination) Len() int {
-	return len(m)
-}
-
-func (m byDestination) Less(i, j int) bool {
-	return m.parts(i) < m.parts(j)
-}
-
-func (m byDestination) Swap(i, j int) {
-	m[i], m[j] = m[j], m[i]
-}
-
-func (m byDestination) parts(i int) int {
-	return strings.Count(filepath.Clean(m[i].Destination), string(os.PathSeparator))
+func parts(m spec.Mount) int {
+	// We must special case a root mount /.
+	// The count of "/" and "/proc" are both 1 but of course logically "/" must
+	// be mounted before "/proc" as such set the count to 0.
+	if m.Destination == "/" {
+		return 0
+	}
+	return strings.Count(filepath.Clean(m.Destination), string(os.PathSeparator))
 }
 
 func sortMounts(m []spec.Mount) []spec.Mount {
-	sort.Sort(byDestination(m))
+	slices.SortStableFunc(m, func(a, b spec.Mount) int {
+		aLen := parts(a)
+		bLen := parts(b)
+		if aLen < bLen {
+			return -1
+		}
+		if aLen == bLen {
+			return 0
+		}
+		return 1
+	})
 	return m
 }
 
@@ -149,7 +151,7 @@ func checkDependencyContainer(depCtr, ctr *Container) error {
 
 // hijackWriteError writes an error to a hijacked HTTP session.
 func hijackWriteError(toWrite error, cid string, terminal bool, httpBuf *bufio.ReadWriter) {
-	if toWrite != nil {
+	if toWrite != nil && !errors.Is(toWrite, define.ErrDetach) {
 		errString := []byte(fmt.Sprintf("Error: %v\n", toWrite))
 		if !terminal {
 			// We need a header.
