@@ -1,3 +1,5 @@
+//go:build !remote
+
 package compat
 
 import (
@@ -23,9 +25,9 @@ import (
 	"github.com/containers/podman/v5/pkg/domain/infra/abi"
 	"github.com/containers/podman/v5/pkg/util"
 	"github.com/containers/storage"
-	docker "github.com/docker/docker/api/types"
 	dockerContainer "github.com/docker/docker/api/types/container"
 	dockerImage "github.com/docker/docker/api/types/image"
+	dockerStorage "github.com/docker/docker/api/types/storage"
 	"github.com/docker/go-connections/nat"
 	"github.com/opencontainers/go-digest"
 	"github.com/sirupsen/logrus"
@@ -137,7 +139,7 @@ func CommitContainer(w http.ResponseWriter, r *http.Request) {
 	options.Author = query.Author
 	options.Pause = query.Pause
 	options.Squash = query.Squash
-	options.Changes = handlers.DecodeChanges(query.Changes)
+	options.Changes = util.DecodeChanges(query.Changes)
 	if r.Body != nil {
 		defer r.Body.Close()
 		if options.CommitOptions.OverrideConfig, err = abi.DecodeOverrideConfig(r.Body); err != nil {
@@ -241,7 +243,7 @@ func CreateImageFromSrc(w http.ResponseWriter, r *http.Request) {
 		Status         string            `json:"status"`
 		Progress       string            `json:"progress"`
 		ProgressDetail map[string]string `json:"progressDetail"`
-		Id             string            `json:"id"` //nolint:revive,stylecheck
+		Id             string            `json:"id"`
 	}{
 		Status:         report.Id,
 		ProgressDetail: map[string]string{},
@@ -369,7 +371,7 @@ func imageDataToImageInspect(ctx context.Context, l *libimage.Image) (*handlers.
 		StopSignal:   info.Config.StopSignal,
 	}
 
-	rootfs := docker.RootFS{}
+	rootfs := dockerImage.RootFS{}
 	if info.RootFS != nil {
 		rootfs.Type = info.RootFS.Type
 		rootfs.Layers = make([]string, 0, len(info.RootFS.Layers))
@@ -378,7 +380,7 @@ func imageDataToImageInspect(ctx context.Context, l *libimage.Image) (*handlers.
 		}
 	}
 
-	graphDriver := docker.GraphDriverData{
+	graphDriver := dockerStorage.DriverData{
 		Name: info.GraphDriver.Name,
 		Data: info.GraphDriver.Data,
 	}
@@ -387,7 +389,7 @@ func imageDataToImageInspect(ctx context.Context, l *libimage.Image) (*handlers.
 	cc.Hostname = info.ID[0:11] // short ID is the hostname
 	cc.Volumes = info.Config.Volumes
 
-	dockerImageInspect := docker.ImageInspect{
+	dockerImageInspect := dockerImage.InspectResponse{
 		Architecture:    info.Architecture,
 		Author:          info.Author,
 		Comment:         info.Comment,
@@ -408,7 +410,7 @@ func imageDataToImageInspect(ctx context.Context, l *libimage.Image) (*handlers.
 		Variant:         "",
 		VirtualSize:     info.VirtualSize,
 	}
-	return &handlers.ImageInspect{ImageInspect: dockerImageInspect}, nil
+	return &handlers.ImageInspect{InspectResponse: dockerImageInspect}, nil
 }
 
 // portsToPortSet converts libpod's exposed ports to docker's structs
