@@ -95,81 +95,83 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 	}
 
 	query := struct {
-		AddHosts                string   `schema:"extrahosts"`
-		AdditionalCapabilities  string   `schema:"addcaps"`
-		AdditionalBuildContexts string   `schema:"additionalbuildcontexts"`
-		AllPlatforms            bool     `schema:"allplatforms"`
-		Annotations             string   `schema:"annotations"`
-		AppArmor                string   `schema:"apparmor"`
-		BuildArgs               string   `schema:"buildargs"`
-		CacheFrom               string   `schema:"cachefrom"`
-		CacheTo                 string   `schema:"cacheto"`
-		CacheTTL                string   `schema:"cachettl"`
-		CgroupParent            string   `schema:"cgroupparent"`
-		CompatVolumes           bool     `schema:"compatvolumes"`
-		Compression             uint64   `schema:"compression"`
-		ConfigureNetwork        string   `schema:"networkmode"`
-		CPPFlags                string   `schema:"cppflags"`
-		CpuPeriod               uint64   `schema:"cpuperiod"`  //nolint:revive,stylecheck
-		CpuQuota                int64    `schema:"cpuquota"`   //nolint:revive,stylecheck
-		CpuSetCpus              string   `schema:"cpusetcpus"` //nolint:revive,stylecheck
-		CpuSetMems              string   `schema:"cpusetmems"` //nolint:revive,stylecheck
-		CpuShares               uint64   `schema:"cpushares"`  //nolint:revive,stylecheck
-		DNSOptions              string   `schema:"dnsoptions"`
-		DNSSearch               string   `schema:"dnssearch"`
-		DNSServers              string   `schema:"dnsservers"`
-		Devices                 string   `schema:"devices"`
-		Dockerfile              string   `schema:"dockerfile"`
-		DropCapabilities        string   `schema:"dropcaps"`
-		Envs                    []string `schema:"setenv"`
-		Excludes                string   `schema:"excludes"`
-		ForceRm                 bool     `schema:"forcerm"`
-		From                    string   `schema:"from"`
-		GroupAdd                []string `schema:"groupadd"`
-		HTTPProxy               bool     `schema:"httpproxy"`
-		IDMappingOptions        string   `schema:"idmappingoptions"`
-		IdentityLabel           bool     `schema:"identitylabel"`
-		Ignore                  bool     `schema:"ignore"`
-		Isolation               string   `schema:"isolation"`
-		Jobs                    int      `schema:"jobs"`
-		LabelOpts               string   `schema:"labelopts"`
-		Labels                  string   `schema:"labels"`
-		LayerLabels             []string `schema:"layerLabel"`
-		Layers                  bool     `schema:"layers"`
-		LogRusage               bool     `schema:"rusage"`
-		Manifest                string   `schema:"manifest"`
-		MemSwap                 int64    `schema:"memswap"`
-		Memory                  int64    `schema:"memory"`
-		NamespaceOptions        string   `schema:"nsoptions"`
-		NoCache                 bool     `schema:"nocache"`
-		OmitHistory             bool     `schema:"omithistory"`
-		OSFeatures              []string `schema:"osfeature"`
-		OSVersion               string   `schema:"osversion"`
-		OutputFormat            string   `schema:"outputformat"`
-		Platform                []string `schema:"platform"`
-		Pull                    bool     `schema:"pull"`
-		PullPolicy              string   `schema:"pullpolicy"`
-		Quiet                   bool     `schema:"q"`
-		Registry                string   `schema:"registry"`
-		Rm                      bool     `schema:"rm"`
-		RusageLogFile           string   `schema:"rusagelogfile"`
-		Remote                  string   `schema:"remote"`
-		Retry                   int      `schema:"retry"`
-		RetryDelay              string   `schema:"retry-delay"`
-		Seccomp                 string   `schema:"seccomp"`
-		Secrets                 string   `schema:"secrets"`
-		SecurityOpt             string   `schema:"securityopt"`
-		ShmSize                 int      `schema:"shmsize"`
-		SkipUnusedStages        bool     `schema:"skipunusedstages"`
-		Squash                  bool     `schema:"squash"`
-		TLSVerify               bool     `schema:"tlsVerify"`
-		Tags                    []string `schema:"t"`
-		Target                  string   `schema:"target"`
-		Timestamp               int64    `schema:"timestamp"`
-		Ulimits                 string   `schema:"ulimits"`
-		UnsetEnvs               []string `schema:"unsetenv"`
-		UnsetLabels             []string `schema:"unsetlabel"`
-		Volumes                 []string `schema:"volume"`
+		AddHosts                string             `schema:"extrahosts"`
+		AdditionalCapabilities  string             `schema:"addcaps"`
+		AdditionalBuildContexts string             `schema:"additionalbuildcontexts"`
+		AllPlatforms            bool               `schema:"allplatforms"`
+		Annotations             string             `schema:"annotations"`
+		AppArmor                string             `schema:"apparmor"`
+		BuildArgs               string             `schema:"buildargs"`
+		CacheFrom               string             `schema:"cachefrom"`
+		CacheTo                 string             `schema:"cacheto"`
+		CacheTTL                string             `schema:"cachettl"`
+		CgroupParent            string             `schema:"cgroupparent"`
+		CompatVolumes           bool               `schema:"compatvolumes"`
+		Compression             uint64             `schema:"compression"`
+		ConfigureNetwork        string             `schema:"networkmode"`
+		CPPFlags                string             `schema:"cppflags"`
+		CpuPeriod               uint64             `schema:"cpuperiod"`
+		CpuQuota                int64              `schema:"cpuquota"`
+		CpuSetCpus              string             `schema:"cpusetcpus"`
+		CpuSetMems              string             `schema:"cpusetmems"`
+		CpuShares               uint64             `schema:"cpushares"`
+		DNSOptions              string             `schema:"dnsoptions"`
+		DNSSearch               string             `schema:"dnssearch"`
+		DNSServers              string             `schema:"dnsservers"`
+		Devices                 string             `schema:"devices"`
+		Dockerfile              string             `schema:"dockerfile"`
+		DropCapabilities        string             `schema:"dropcaps"`
+		Envs                    []string           `schema:"setenv"`
+		Excludes                string             `schema:"excludes"`
+		ForceRm                 bool               `schema:"forcerm"`
+		From                    string             `schema:"from"`
+		GroupAdd                []string           `schema:"groupadd"`
+		HTTPProxy               bool               `schema:"httpproxy"`
+		IDMappingOptions        string             `schema:"idmappingoptions"`
+		IdentityLabel           bool               `schema:"identitylabel"`
+		Ignore                  bool               `schema:"ignore"`
+		InheritLabels           types.OptionalBool `schema:"inheritlabels"`
+		Isolation               string             `schema:"isolation"`
+		Jobs                    int                `schema:"jobs"`
+		LabelOpts               string             `schema:"labelopts"`
+		Labels                  string             `schema:"labels"`
+		LayerLabels             []string           `schema:"layerLabel"`
+		Layers                  bool               `schema:"layers"`
+		LogRusage               bool               `schema:"rusage"`
+		Manifest                string             `schema:"manifest"`
+		MemSwap                 int64              `schema:"memswap"`
+		Memory                  int64              `schema:"memory"`
+		NamespaceOptions        string             `schema:"nsoptions"`
+		NoCache                 bool               `schema:"nocache"`
+		NoHosts                 bool               `schema:"nohosts"`
+		OmitHistory             bool               `schema:"omithistory"`
+		OSFeatures              []string           `schema:"osfeature"`
+		OSVersion               string             `schema:"osversion"`
+		OutputFormat            string             `schema:"outputformat"`
+		Platform                []string           `schema:"platform"`
+		Pull                    bool               `schema:"pull"`
+		PullPolicy              string             `schema:"pullpolicy"`
+		Quiet                   bool               `schema:"q"`
+		Registry                string             `schema:"registry"`
+		Rm                      bool               `schema:"rm"`
+		RusageLogFile           string             `schema:"rusagelogfile"`
+		Remote                  string             `schema:"remote"`
+		Retry                   int                `schema:"retry"`
+		RetryDelay              string             `schema:"retry-delay"`
+		Seccomp                 string             `schema:"seccomp"`
+		Secrets                 string             `schema:"secrets"`
+		SecurityOpt             string             `schema:"securityopt"`
+		ShmSize                 int                `schema:"shmsize"`
+		SkipUnusedStages        bool               `schema:"skipunusedstages"`
+		Squash                  bool               `schema:"squash"`
+		TLSVerify               bool               `schema:"tlsVerify"`
+		Tags                    []string           `schema:"t"`
+		Target                  string             `schema:"target"`
+		Timestamp               int64              `schema:"timestamp"`
+		Ulimits                 string             `schema:"ulimits"`
+		UnsetEnvs               []string           `schema:"unsetenv"`
+		UnsetLabels             []string           `schema:"unsetlabel"`
+		Volumes                 []string           `schema:"volume"`
 	}{
 		Dockerfile:       "Dockerfile",
 		IdentityLabel:    true,
@@ -257,7 +259,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 
 			for _, containerfile := range m {
 				// Add path to containerfile iff it is not URL
-				if !(strings.HasPrefix(containerfile, "http://") || strings.HasPrefix(containerfile, "https://")) {
+				if !strings.HasPrefix(containerfile, "http://") && !strings.HasPrefix(containerfile, "https://") {
 					containerfile = filepath.Join(contextDirectory,
 						filepath.Clean(filepath.FromSlash(containerfile)))
 				}
@@ -720,6 +722,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 			LabelOpts:          labelOpts,
 			Memory:             query.Memory,
 			MemorySwap:         query.MemSwap,
+			NoHosts:            query.NoHosts,
 			OmitHistory:        query.OmitHistory,
 			SeccompProfilePath: seccomp,
 			ShmSize:            strconv.Itoa(query.ShmSize),
@@ -742,6 +745,7 @@ func BuildImage(w http.ResponseWriter, r *http.Request) {
 		IDMappingOptions:               &idMappingOptions,
 		IgnoreUnrecognizedInstructions: query.Ignore,
 		IgnoreFile:                     ignoreFile,
+		InheritLabels:                  query.InheritLabels,
 		Isolation:                      isolation,
 		Jobs:                           &jobs,
 		Labels:                         labels,
