@@ -19,7 +19,6 @@ import (
 	"github.com/containers/podman/v5/libpod"
 	api "github.com/containers/podman/v5/pkg/api/types"
 	"github.com/containers/podman/v5/pkg/errorhandling"
-	"github.com/containers/podman/v5/pkg/util"
 	"github.com/containers/storage"
 	"github.com/docker/distribution/registry/api/errcode"
 	"github.com/docker/docker/pkg/jsonmessage"
@@ -30,7 +29,11 @@ import (
 // request is for the compat API and if containers.conf set the specific mode.
 // If nameOrID is a (short) ID for a local image, the full ID will be returned.
 func NormalizeToDockerHub(r *http.Request, nameOrID string) (string, error) {
-	if IsLibpodRequest(r) || !util.DefaultContainerConfig().Engine.CompatAPIEnforceDockerHub {
+	cfg, err := config.Default()
+	if err != nil {
+		return "", err
+	}
+	if IsLibpodRequest(r) || !cfg.Engine.CompatAPIEnforceDockerHub {
 		return nameOrID, nil
 	}
 
@@ -62,11 +65,16 @@ func NormalizeToDockerHub(r *http.Request, nameOrID string) (string, error) {
 // PossiblyEnforceDockerHub sets fields in the system context to enforce
 // resolving short names to Docker Hub if the request is for the compat API and
 // if containers.conf set the specific mode.
-func PossiblyEnforceDockerHub(r *http.Request, sys *types.SystemContext) {
-	if IsLibpodRequest(r) || !util.DefaultContainerConfig().Engine.CompatAPIEnforceDockerHub {
-		return
+func PossiblyEnforceDockerHub(r *http.Request, sys *types.SystemContext) error {
+	cfg, err := config.Default()
+	if err != nil {
+		return err
+	}
+	if IsLibpodRequest(r) || !cfg.Engine.CompatAPIEnforceDockerHub {
+		return nil
 	}
 	sys.PodmanOnlyShortNamesIgnoreRegistriesConfAndForceDockerHub = true
+	return nil
 }
 
 // IsRegistryReference checks if the specified name points to the "docker://"
@@ -161,6 +169,7 @@ loop: // break out of for/select infinite loop
 				report.Status = "Downloading"
 				report.Progress.Current = int64(e.Offset)
 				report.Progress.Total = e.Artifact.Size
+				//nolint:staticcheck // Deprecated field, but because consumers might still read it keep it.
 				report.ProgressMessage = report.Progress.String()
 			case types.ProgressEventSkipped:
 				report.Status = "Already exists"
@@ -185,6 +194,7 @@ loop: // break out of for/select infinite loop
 				report.Error = &jsonmessage.JSONError{
 					Message: msg,
 				}
+				//nolint:staticcheck // Deprecated field, but because consumers might still read it keep it.
 				report.ErrorMessage = msg
 			} else {
 				pulledImages := pullRes.images
@@ -197,6 +207,7 @@ loop: // break out of for/select infinite loop
 					report.Error = &jsonmessage.JSONError{
 						Message: msg,
 					}
+					//nolint:staticcheck // Deprecated field, but because consumers might still read it keep it.
 					report.ErrorMessage = msg
 					writeStatusCode(http.StatusInternalServerError)
 				}
