@@ -9,7 +9,6 @@ import (
 	entitiesTypes "github.com/containers/podman/v5/pkg/domain/entities/types"
 	"github.com/containers/podman/v5/pkg/specgen"
 	"github.com/containers/storage/pkg/archive"
-	dockerAPI "github.com/docker/docker/api/types"
 )
 
 type Container struct {
@@ -21,7 +20,7 @@ type Volume struct {
 }
 
 type Report struct {
-	Id  []string //nolint:revive,stylecheck
+	Id  []string
 	Err map[string]error
 }
 
@@ -40,6 +39,7 @@ type NetFlags struct {
 	MacAddr      string   `json:"mac-address,omitempty"`
 	Publish      []string `json:"publish,omitempty"`
 	IP           string   `json:"ip,omitempty"`
+	NoHostname   bool     `json:"no-hostname,omitempty"`
 	NoHosts      bool     `json:"no-hosts,omitempty"`
 	Network      string   `json:"network,omitempty"`
 	NetworkAlias []string `json:"network-alias,omitempty"`
@@ -55,7 +55,9 @@ type NetOptions struct {
 	DNSOptions         []string                           `json:"dns_option,omitempty"`
 	DNSSearch          []string                           `json:"dns_search,omitempty"`
 	DNSServers         []net.IP                           `json:"dns_server,omitempty"`
+	HostsFile          string                             `json:"hosts_file,omitempty"`
 	Network            specgen.Namespace                  `json:"netns,omitempty"`
+	NoHostname         bool                               `json:"no_manage_hostname,omitempty"`
 	NoHosts            bool                               `json:"no_manage_hosts,omitempty"`
 	PublishPorts       []types.PortMapping                `json:"portmappings,omitempty"`
 	// NetworkOptions are additional options for each network
@@ -114,5 +116,4 @@ type IDOrNameResponse struct {
 	IDOrName string
 }
 
-// swagger:model
-type IDResponse dockerAPI.IDResponse
+type IDResponse = entitiesTypes.IDResponse
