@@ -14,6 +14,7 @@ import (
 	"github.com/containers/podman/v5/libpod/define"
 	"github.com/containers/podman/v5/pkg/rootless"
 	"github.com/containers/podman/v5/pkg/util"
+	"github.com/containers/storage/pkg/fileutils"
 
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
@@ -23,16 +24,19 @@ import (
 )
 
 // DevicesFromPath computes a list of devices
-func DevicesFromPath(g *generate.Generator, devicePath string) error {
+func DevicesFromPath(g *generate.Generator, devicePath string, config *config.Config) error {
 	if isCDIDevice(devicePath) {
-		registry := cdi.GetRegistry(
+		registry, err := cdi.NewCache(
+			cdi.WithSpecDirs(config.Engine.CdiSpecDirs.Get()...),
 			cdi.WithAutoRefresh(false),
 		)
+		if err != nil {
+			return fmt.Errorf("creating CDI registry: %w", err)
+		}
 		if err := registry.Refresh(); err != nil {
 			logrus.Debugf("The following error was triggered when refreshing the CDI registry: %v", err)
 		}
-		_, err := registry.InjectDevices(g.Config, devicePath)
-		if err != nil {
+		if _, err := registry.InjectDevices(g.Config, devicePath); err != nil {
 			return fmt.Errorf("setting up CDI devices: %w", err)
 		}
 		return nil
@@ -94,7 +98,7 @@ func DevicesFromPath(g *generate.Generator, devicePath string) error {
 
 func BlockAccessToKernelFilesystems(privileged, pidModeIsHost bool, mask, unmask []string, g *generate.Generator) {
 	if !privileged {
-		for _, mp := range config.DefaultMaskedPaths {
+		for _, mp := range config.DefaultMaskedPaths() {
 			// check that the path to mask is not in the list of paths to unmask
 			if shouldMask(mp, unmask) {
 				g.AddLinuxMaskedPaths(mp)
@@ -131,7 +135,7 @@ func addDevice(g *generate.Generator, device string) error {
 		return fmt.Errorf("%s is not a valid device: %w", src, err)
 	}
 	if rootless.IsRootless() {
-		if _, err := os.Stat(src); err != nil {
+		if err := fileutils.Exists(src); err != nil {
 			return err
 		}
 		perm := "ro"
@@ -173,7 +177,7 @@ func shouldMask(mask string, unmask []string) bool {
 		for _, m1 := range strings.Split(m, ":") {
 			match, err := filepath.Match(m1, mask)
 			if err != nil {
-				logrus.Errorf(err.Error())
+				logrus.Error(err.Error())
 			}
 			if match {
 				return false
