@@ -11,14 +11,14 @@ import (
 
 	"github.com/containers/common/pkg/config"
 	"github.com/containers/image/v5/manifest"
-	"github.com/containers/podman/v4/cmd/podman/parse"
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/pkg/domain/entities"
-	envLib "github.com/containers/podman/v4/pkg/env"
-	"github.com/containers/podman/v4/pkg/namespaces"
-	"github.com/containers/podman/v4/pkg/specgen"
-	systemdDefine "github.com/containers/podman/v4/pkg/systemd/define"
-	"github.com/containers/podman/v4/pkg/util"
+	"github.com/containers/podman/v5/cmd/podman/parse"
+	"github.com/containers/podman/v5/libpod/define"
+	"github.com/containers/podman/v5/pkg/domain/entities"
+	envLib "github.com/containers/podman/v5/pkg/env"
+	"github.com/containers/podman/v5/pkg/namespaces"
+	"github.com/containers/podman/v5/pkg/specgen"
+	systemdDefine "github.com/containers/podman/v5/pkg/systemd/define"
+	"github.com/containers/podman/v5/pkg/util"
 	"github.com/docker/go-units"
 	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/selinux/go-selinux"
@@ -222,10 +222,12 @@ func setNamespaces(rtc *config.Config, s *specgen.SpecGenerator, c *entities.Con
 		}
 	}
 	userns := c.UserNS
-	if userns == "" && c.Pod == "" {
+	// caller must make sure s.Pod is set before calling this function.
+	if userns == "" && s.Pod == "" {
 		if ns, ok := os.LookupEnv("PODMAN_USERNS"); ok {
 			userns = ns
 		} else {
+			// TODO: This should be moved into pkg/specgen/generate so we don't use the client's containers.conf
 			userns = rtc.Containers.UserNS
 		}
 	}
@@ -326,6 +328,7 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		return err
 	}
 
+	// TODO: This needs to move into pkg/specgen/generate so we aren't using containers.conf on the client.
 	if rtc.Containers.EnableLabeledUsers {
 		defSecurityOpts, err := currentLabelOpts()
 		if err != nil {
@@ -334,6 +337,7 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 
 		c.SecurityOpt = append(defSecurityOpts, c.SecurityOpt...)
 	}
+
 	// validate flags as needed
 	if err := validate(c); err != nil {
 		return err
@@ -350,7 +354,7 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		if c.NoHealthCheck {
 			return errors.New("cannot specify both --no-healthcheck and --health-cmd")
 		}
-		s.HealthConfig, err = makeHealthCheckFromCli(c.HealthCmd, c.HealthInterval, c.HealthRetries, c.HealthTimeout, c.HealthStartPeriod, false)
+		s.HealthConfig, err = MakeHealthCheckFromCli(c.HealthCmd, c.HealthInterval, c.HealthRetries, c.HealthTimeout, c.HealthStartPeriod, false)
 		if err != nil {
 			return err
 		}
@@ -366,6 +370,12 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	}
 	s.HealthCheckOnFailureAction = onFailureAction
 
+	s.HealthLogDestination = c.HealthLogDestination
+
+	s.HealthMaxLogCount = c.HealthMaxLogCount
+
+	s.HealthMaxLogSize = c.HealthMaxLogSize
+
 	if c.StartupHCCmd != "" {
 		if c.NoHealthCheck {
 			return errors.New("cannot specify both --no-healthcheck and --health-startup-cmd")
@@ -373,7 +383,7 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		// The hardcoded "1s" will be discarded, as the startup
 		// healthcheck does not have a period. So just hardcode
 		// something that parses correctly.
-		tmpHcConfig, err := makeHealthCheckFromCli(c.StartupHCCmd, c.StartupHCInterval, c.StartupHCRetries, c.StartupHCTimeout, "1s", true)
+		tmpHcConfig, err := MakeHealthCheckFromCli(c.StartupHCCmd, c.StartupHCInterval, c.StartupHCRetries, c.StartupHCTimeout, "1s", true)
 		if err != nil {
 			return err
 		}
@@ -385,26 +395,6 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.StartupHealthConfig.Successes = int(c.StartupHCSuccesses)
 	}
 
-	if err := setNamespaces(rtc, s, c); err != nil {
-		return err
-	}
-
-	if !s.Terminal {
-		s.Terminal = c.TTY
-	}
-
-	if err := verifyExpose(c.Expose); err != nil {
-		return err
-	}
-	// We are not handling the Expose flag yet.
-	// s.PortsExpose = c.Expose
-	if c.Net != nil {
-		s.PortMappings = c.Net.PublishPorts
-	}
-	if !s.PublishExposedPorts {
-		s.PublishExposedPorts = c.PublishAll
-	}
-
 	if len(s.Pod) == 0 || len(c.Pod) > 0 {
 		s.Pod = c.Pod
 	}
@@ -420,6 +410,27 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.Pod = podID
 	}
 
+	// Important s.Pod must be set above here.
+	if err := setNamespaces(rtc, s, c); err != nil {
+		return err
+	}
+
+	if s.Terminal == nil {
+		s.Terminal = &c.TTY
+	}
+
+	if err := verifyExpose(c.Expose); err != nil {
+		return err
+	}
+	// We are not handling the Expose flag yet.
+	// s.PortsExpose = c.Expose
+	if c.Net != nil {
+		s.PortMappings = c.Net.PublishPorts
+	}
+	if s.PublishExposedPorts == nil {
+		s.PublishExposedPorts = &c.PublishAll
+	}
+
 	expose, err := CreateExpose(c.Expose)
 	if err != nil {
 		return err
@@ -450,12 +461,12 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	// any case.
 	osEnv := envLib.Map(os.Environ())
 
-	if !s.EnvHost {
-		s.EnvHost = c.EnvHost
+	if s.EnvHost == nil {
+		s.EnvHost = &c.EnvHost
 	}
 
-	if !s.HTTPProxy {
-		s.HTTPProxy = c.HTTPProxy
+	if s.HTTPProxy == nil {
+		s.HTTPProxy = &c.HTTPProxy
 	}
 
 	// env-file overrides any previous variables
@@ -491,29 +502,43 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.Labels = labels
 	}
 
+	// Intel RDT CAT
+	if c.IntelRdtClosID != "" {
+		s.IntelRdt = &specs.LinuxIntelRdt{}
+		s.IntelRdt.ClosID = c.IntelRdtClosID
+	}
+
 	// ANNOTATIONS
 	annotations := make(map[string]string)
 
 	// Last, add user annotations
 	for _, annotation := range c.Annotation {
-		splitAnnotation := strings.SplitN(annotation, "=", 2)
-		if len(splitAnnotation) < 2 {
+		key, val, hasVal := strings.Cut(annotation, "=")
+		if !hasVal {
 			return errors.New("annotations must be formatted KEY=VALUE")
 		}
-		annotations[splitAnnotation[0]] = splitAnnotation[1]
+		annotations[key] = val
 	}
 	if len(s.Annotations) == 0 {
 		s.Annotations = annotations
 	}
+	// Add the user namespace configuration to the annotations
+	if c.UserNS != "" {
+		s.Annotations[define.UserNsAnnotation] = c.UserNS
+	}
+
+	if c.PIDsLimit != nil {
+		s.Annotations[define.PIDsLimitAnnotation] = strconv.FormatInt(*c.PIDsLimit, 10)
+	}
 
 	if len(c.StorageOpts) > 0 {
 		opts := make(map[string]string, len(c.StorageOpts))
 		for _, opt := range c.StorageOpts {
-			split := strings.SplitN(opt, "=", 2)
-			if len(split) != 2 {
+			key, val, hasVal := strings.Cut(opt, "=")
+			if !hasVal {
 				return errors.New("storage-opt must be formatted KEY=VALUE")
 			}
-			opts[split[0]] = split[1]
+			opts[key] = val
 		}
 		s.StorageOpts = opts
 	}
@@ -529,12 +554,6 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.Entrypoint = entrypoint
 	}
 
-	// Include the command used to create the container.
-
-	if len(s.ContainerCreateCommand) == 0 {
-		s.ContainerCreateCommand = os.Args
-	}
-
 	if len(inputCommand) > 0 {
 		s.Command = inputCommand
 	}
@@ -566,12 +585,14 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 
 	if c.Net != nil {
 		s.HostAdd = c.Net.AddHosts
-		s.UseImageResolvConf = c.Net.UseImageResolvConf
+		s.BaseHostsFile = c.Net.HostsFile
+		s.UseImageResolvConf = &c.Net.UseImageResolvConf
 		s.DNSServers = c.Net.DNSServers
 		s.DNSSearch = c.Net.DNSSearch
 		s.DNSOptions = c.Net.DNSOptions
 		s.NetworkOptions = c.Net.NetworkOptions
-		s.UseImageHosts = c.Net.NoHosts
+		s.UseImageHostname = &c.Net.NoHostname
+		s.UseImageHosts = &c.Net.NoHosts
 	}
 	if len(s.HostUsers) == 0 || len(c.HostUsers) != 0 {
 		s.HostUsers = c.HostUsers
@@ -581,9 +602,6 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 			s.ImageVolumeMode = c.ImageVolume
 		}
 	}
-	if len(s.ImageVolumeMode) == 0 {
-		s.ImageVolumeMode = rtc.Engine.ImageVolumeMode
-	}
 	if s.ImageVolumeMode == define.TypeBind {
 		s.ImageVolumeMode = "anonymous"
 	}
@@ -616,9 +634,6 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	if len(s.CgroupsMode) == 0 {
 		s.CgroupsMode = c.CgroupsMode
 	}
-	if s.CgroupsMode == "" {
-		s.CgroupsMode = rtc.Cgroups()
-	}
 
 	if len(s.Groups) == 0 || len(c.GroupAdd) != 0 {
 		s.Groups = c.GroupAdd
@@ -644,11 +659,11 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	if len(s.CapDrop) == 0 || len(c.CapDrop) != 0 {
 		s.CapDrop = c.CapDrop
 	}
-	if !s.Privileged {
-		s.Privileged = c.Privileged
+	if s.Privileged == nil {
+		s.Privileged = &c.Privileged
 	}
-	if !s.ReadOnlyFilesystem {
-		s.ReadOnlyFilesystem = c.ReadOnly
+	if s.ReadOnlyFilesystem == nil {
+		s.ReadOnlyFilesystem = &c.ReadOnly
 	}
 	if len(s.ConmonPidFile) == 0 || len(c.ConmonPIDFile) != 0 {
 		s.ConmonPidFile = c.ConmonPIDFile
@@ -661,17 +676,18 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	// Only add ReadWrite tmpfs mounts iff the container is
 	// being run ReadOnly and ReadWriteTmpFS is not disabled,
 	// (user specifying --read-only-tmpfs=false.)
-	s.ReadWriteTmpfs = c.ReadOnly && c.ReadWriteTmpFS
+	localRWTmpfs := c.ReadOnly && c.ReadWriteTmpFS
+	s.ReadWriteTmpfs = &localRWTmpfs
 
 	//  TODO convert to map?
 	// check if key=value and convert
 	sysmap := make(map[string]string)
 	for _, ctl := range c.Sysctl {
-		splitCtl := strings.SplitN(ctl, "=", 2)
-		if len(splitCtl) < 2 {
+		key, val, hasVal := strings.Cut(ctl, "=")
+		if !hasVal {
 			return fmt.Errorf("invalid sysctl value %q", ctl)
 		}
-		sysmap[splitCtl[0]] = splitCtl[1]
+		sysmap[key] = val
 	}
 	if len(s.Sysctl) == 0 || len(c.Sysctl) != 0 {
 		s.Sysctl = sysmap
@@ -684,53 +700,57 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	for _, opt := range c.SecurityOpt {
 		// Docker deprecated the ":" syntax but still supports it,
 		// so we need to as well
-		var con []string
+		var key, val string
+		var hasVal bool
 		if strings.Contains(opt, "=") {
-			con = strings.SplitN(opt, "=", 2)
+			key, val, hasVal = strings.Cut(opt, "=")
 		} else {
-			con = strings.SplitN(opt, ":", 2)
+			key, val, hasVal = strings.Cut(opt, ":")
 		}
-		if len(con) != 2 &&
-			con[0] != "no-new-privileges" {
+		if !hasVal &&
+			key != "no-new-privileges" {
 			return fmt.Errorf("invalid --security-opt 1: %q", opt)
 		}
-		switch con[0] {
+		switch key {
 		case "apparmor":
-			s.ContainerSecurityConfig.ApparmorProfile = con[1]
-			s.Annotations[define.InspectAnnotationApparmor] = con[1]
+			s.ContainerSecurityConfig.ApparmorProfile = val
+			s.Annotations[define.InspectAnnotationApparmor] = val
 		case "label":
-			if con[1] == "nested" {
-				s.ContainerSecurityConfig.LabelNested = true
+			if val == "nested" {
+				localTrue := true
+				s.ContainerSecurityConfig.LabelNested = &localTrue
 				continue
 			}
 			// TODO selinux opts and label opts are the same thing
-			s.ContainerSecurityConfig.SelinuxOpts = append(s.ContainerSecurityConfig.SelinuxOpts, con[1])
+			s.ContainerSecurityConfig.SelinuxOpts = append(s.ContainerSecurityConfig.SelinuxOpts, val)
 			s.Annotations[define.InspectAnnotationLabel] = strings.Join(s.ContainerSecurityConfig.SelinuxOpts, ",label=")
 		case "mask":
-			s.ContainerSecurityConfig.Mask = append(s.ContainerSecurityConfig.Mask, strings.Split(con[1], ":")...)
+			s.ContainerSecurityConfig.Mask = append(s.ContainerSecurityConfig.Mask, strings.Split(val, ":")...)
 		case "proc-opts":
-			s.ProcOpts = strings.Split(con[1], ",")
+			s.ProcOpts = strings.Split(val, ",")
 		case "seccomp":
-			s.SeccompProfilePath = con[1]
-			s.Annotations[define.InspectAnnotationSeccomp] = con[1]
+			s.SeccompProfilePath = val
+			s.Annotations[define.InspectAnnotationSeccomp] = val
 			// this option is for docker compatibility, it is the same as unmask=ALL
 		case "systempaths":
-			if con[1] == "unconfined" {
+			if val == "unconfined" {
 				s.ContainerSecurityConfig.Unmask = append(s.ContainerSecurityConfig.Unmask, []string{"ALL"}...)
 			} else {
-				return fmt.Errorf("invalid systempaths option %q, only `unconfined` is supported", con[1])
+				return fmt.Errorf("invalid systempaths option %q, only `unconfined` is supported", val)
 			}
 		case "unmask":
-			s.ContainerSecurityConfig.Unmask = append(s.ContainerSecurityConfig.Unmask, con[1:]...)
+			if hasVal {
+				s.ContainerSecurityConfig.Unmask = append(s.ContainerSecurityConfig.Unmask, val)
+			}
 		case "no-new-privileges":
 			noNewPrivileges := true
-			if len(con) == 2 {
-				noNewPrivileges, err = strconv.ParseBool(con[1])
+			if hasVal {
+				noNewPrivileges, err = strconv.ParseBool(val)
 				if err != nil {
 					return fmt.Errorf("invalid --security-opt 2: %q", opt)
 				}
 			}
-			s.ContainerSecurityConfig.NoNewPrivileges = noNewPrivileges
+			s.ContainerSecurityConfig.NoNewPrivileges = &noNewPrivileges
 		default:
 			return fmt.Errorf("invalid --security-opt 2: %q", opt)
 		}
@@ -746,18 +766,18 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 
 	// Only add read-only tmpfs mounts in case that we are read-only and the
 	// read-only tmpfs flag has been set.
-	mounts, volumes, overlayVolumes, imageVolumes, err := parseVolumes(rtc, c.Volume, c.Mount, c.TmpFS)
+	containerMounts, err := parseVolumes(rtc, c.Volume, c.Mount, c.TmpFS)
 	if err != nil {
 		return err
 	}
 	if len(s.Mounts) == 0 || len(c.Mount) != 0 {
-		s.Mounts = mounts
+		s.Mounts = containerMounts.mounts
 	}
 	if len(s.Volumes) == 0 || len(c.Volume) != 0 {
-		s.Volumes = volumes
+		s.Volumes = containerMounts.volumes
 	}
 
-	if s.ContainerSecurityConfig.LabelNested {
+	if s.LabelNested != nil && *s.LabelNested {
 		// Need to unmask the SELinux file system
 		s.Unmask = append(s.Unmask, "/sys/fs/selinux", "/proc")
 		s.Mounts = append(s.Mounts, specs.Mount{
@@ -769,13 +789,21 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	}
 	// TODO make sure these work in clone
 	if len(s.OverlayVolumes) == 0 {
-		s.OverlayVolumes = overlayVolumes
+		s.OverlayVolumes = containerMounts.overlayVolumes
 	}
 	if len(s.ImageVolumes) == 0 {
-		s.ImageVolumes = imageVolumes
+		s.ImageVolumes = containerMounts.imageVolumes
+	}
+	if len(s.ArtifactVolumes) == 0 {
+		s.ArtifactVolumes = containerMounts.artifactVolumes
 	}
 
-	for _, dev := range c.Devices {
+	devices := c.Devices
+	for _, gpu := range c.GPUs {
+		devices = append(devices, "nvidia.com/gpu="+gpu)
+	}
+
+	for _, dev := range devices {
 		s.Devices = append(s.Devices, specs.LinuxDevice{Path: dev})
 	}
 
@@ -787,14 +815,14 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.DeviceCgroupRule = append(s.DeviceCgroupRule, dev)
 	}
 
-	if !s.Init {
-		s.Init = c.Init
+	if s.Init == nil {
+		s.Init = &c.Init
 	}
 	if len(s.InitPath) == 0 || len(c.InitPath) != 0 {
 		s.InitPath = c.InitPath
 	}
-	if !s.Stdin {
-		s.Stdin = c.Interactive
+	if s.Stdin == nil {
+		s.Stdin = &c.Interactive
 	}
 	// quiet
 	// DeviceCgroupRules: c.StringSlice("device-cgroup-rule"),
@@ -807,23 +835,23 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 
 	logOpts := make(map[string]string)
 	for _, o := range c.LogOptions {
-		split := strings.SplitN(o, "=", 2)
-		if len(split) < 2 {
+		key, val, hasVal := strings.Cut(o, "=")
+		if !hasVal {
 			return fmt.Errorf("invalid log option %q", o)
 		}
-		switch strings.ToLower(split[0]) {
+		switch strings.ToLower(key) {
 		case "driver":
-			s.LogConfiguration.Driver = split[1]
+			s.LogConfiguration.Driver = val
 		case "path":
-			s.LogConfiguration.Path = split[1]
+			s.LogConfiguration.Path = val
 		case "max-size":
-			logSize, err := units.FromHumanSize(split[1])
+			logSize, err := units.FromHumanSize(val)
 			if err != nil {
 				return err
 			}
 			s.LogConfiguration.Size = logSize
 		default:
-			logOpts[split[0]] = split[1]
+			logOpts[key] = val
 		}
 	}
 	if len(s.LogConfiguration.Options) == 0 || len(c.LogOptions) != 0 {
@@ -832,9 +860,17 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	if len(s.Name) == 0 || len(c.Name) != 0 {
 		s.Name = c.Name
 	}
+
+	if c.PreserveFDs != 0 && c.PreserveFD != nil {
+		return errors.New("cannot specify both --preserve-fds and --preserve-fd")
+	}
+
 	if s.PreserveFDs == 0 || c.PreserveFDs != 0 {
 		s.PreserveFDs = c.PreserveFDs
 	}
+	if s.PreserveFD == nil || c.PreserveFD != nil {
+		s.PreserveFD = c.PreserveFD
+	}
 
 	if s.OOMScoreAdj == nil || c.OOMScoreAdj != nil {
 		s.OOMScoreAdj = c.OOMScoreAdj
@@ -860,8 +896,8 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 		s.Personality.Domain = specs.LinuxPersonalityDomain(c.Personality)
 	}
 
-	if !s.Remove {
-		s.Remove = c.Rm
+	if s.Remove == nil {
+		s.Remove = &c.Rm
 	}
 	if s.StopTimeout == nil || c.StopTimeout != 0 {
 		s.StopTimeout = &c.StopTimeout
@@ -878,8 +914,8 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	if len(s.PidFile) == 0 || len(c.PidFile) != 0 {
 		s.PidFile = c.PidFile
 	}
-	if !s.Volatile {
-		s.Volatile = c.Rm
+	if s.Volatile == nil {
+		s.Volatile = &c.Rm
 	}
 	if len(s.EnvMerge) == 0 || len(c.EnvMerge) != 0 {
 		s.EnvMerge = c.EnvMerge
@@ -887,8 +923,8 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	if len(s.UnsetEnv) == 0 || len(c.UnsetEnv) != 0 {
 		s.UnsetEnv = c.UnsetEnv
 	}
-	if !s.UnsetEnvAll {
-		s.UnsetEnvAll = c.UnsetEnvAll
+	if s.UnsetEnvAll == nil {
+		s.UnsetEnvAll = &c.UnsetEnvAll
 	}
 	if len(s.ChrootDirs) == 0 || len(c.ChrootDirs) != 0 {
 		s.ChrootDirs = c.ChrootDirs
@@ -915,7 +951,7 @@ func FillOutSpecGen(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions
 	return nil
 }
 
-func makeHealthCheckFromCli(inCmd, interval string, retries uint, timeout, startPeriod string, isStartup bool) (*manifest.Schema2HealthConfig, error) {
+func MakeHealthCheckFromCli(inCmd, interval string, retries uint, timeout, startPeriod string, isStartup bool) (*manifest.Schema2HealthConfig, error) {
 	cmdArr := []string{}
 	isArr := true
 	err := json.Unmarshal([]byte(inCmd), &cmdArr) // array unmarshalling
@@ -984,29 +1020,28 @@ func makeHealthCheckFromCli(inCmd, interval string, retries uint, timeout, start
 		return nil, errors.New("healthcheck-start-period must be 0 seconds or greater")
 	}
 	hc.StartPeriod = startPeriodDuration
-
 	return &hc, nil
 }
 
 func parseWeightDevices(weightDevs []string) (map[string]specs.LinuxWeightDevice, error) {
 	wd := make(map[string]specs.LinuxWeightDevice)
-	for _, val := range weightDevs {
-		split := strings.SplitN(val, ":", 2)
-		if len(split) != 2 {
-			return nil, fmt.Errorf("bad format: %s", val)
+	for _, dev := range weightDevs {
+		key, val, hasVal := strings.Cut(dev, ":")
+		if !hasVal {
+			return nil, fmt.Errorf("bad format: %s", dev)
 		}
-		if !strings.HasPrefix(split[0], "/dev/") {
-			return nil, fmt.Errorf("bad format for device path: %s", val)
+		if !strings.HasPrefix(key, "/dev/") {
+			return nil, fmt.Errorf("bad format for device path: %s", dev)
 		}
-		weight, err := strconv.ParseUint(split[1], 10, 0)
+		weight, err := strconv.ParseUint(val, 10, 0)
 		if err != nil {
-			return nil, fmt.Errorf("invalid weight for device: %s", val)
+			return nil, fmt.Errorf("invalid weight for device: %s", dev)
 		}
 		if weight > 0 && (weight < 10 || weight > 1000) {
-			return nil, fmt.Errorf("invalid weight for device: %s", val)
+			return nil, fmt.Errorf("invalid weight for device: %s", dev)
 		}
 		w := uint16(weight)
-		wd[split[0]] = specs.LinuxWeightDevice{
+		wd[key] = specs.LinuxWeightDevice{
 			Weight:     &w,
 			LeafWeight: nil,
 		}
@@ -1016,41 +1051,41 @@ func parseWeightDevices(weightDevs []string) (map[string]specs.LinuxWeightDevice
 
 func parseThrottleBPSDevices(bpsDevices []string) (map[string]specs.LinuxThrottleDevice, error) {
 	td := make(map[string]specs.LinuxThrottleDevice)
-	for _, val := range bpsDevices {
-		split := strings.SplitN(val, ":", 2)
-		if len(split) != 2 {
-			return nil, fmt.Errorf("bad format: %s", val)
+	for _, dev := range bpsDevices {
+		key, val, hasVal := strings.Cut(dev, ":")
+		if !hasVal {
+			return nil, fmt.Errorf("bad format: %s", dev)
 		}
-		if !strings.HasPrefix(split[0], "/dev/") {
-			return nil, fmt.Errorf("bad format for device path: %s", val)
+		if !strings.HasPrefix(key, "/dev/") {
+			return nil, fmt.Errorf("bad format for device path: %s", dev)
 		}
-		rate, err := units.RAMInBytes(split[1])
+		rate, err := units.RAMInBytes(val)
 		if err != nil {
-			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>[<unit>]. Number must be a positive integer. Unit is optional and can be kb, mb, or gb", val)
+			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>[<unit>]. Number must be a positive integer. Unit is optional and can be kb, mb, or gb", dev)
 		}
 		if rate < 0 {
-			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>[<unit>]. Number must be a positive integer. Unit is optional and can be kb, mb, or gb", val)
+			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>[<unit>]. Number must be a positive integer. Unit is optional and can be kb, mb, or gb", dev)
 		}
-		td[split[0]] = specs.LinuxThrottleDevice{Rate: uint64(rate)}
+		td[key] = specs.LinuxThrottleDevice{Rate: uint64(rate)}
 	}
 	return td, nil
 }
 
 func parseThrottleIOPsDevices(iopsDevices []string) (map[string]specs.LinuxThrottleDevice, error) {
 	td := make(map[string]specs.LinuxThrottleDevice)
-	for _, val := range iopsDevices {
-		split := strings.SplitN(val, ":", 2)
-		if len(split) != 2 {
-			return nil, fmt.Errorf("bad format: %s", val)
+	for _, dev := range iopsDevices {
+		key, val, hasVal := strings.Cut(dev, ":")
+		if !hasVal {
+			return nil, fmt.Errorf("bad format: %s", dev)
 		}
-		if !strings.HasPrefix(split[0], "/dev/") {
-			return nil, fmt.Errorf("bad format for device path: %s", val)
+		if !strings.HasPrefix(key, "/dev/") {
+			return nil, fmt.Errorf("bad format for device path: %s", dev)
 		}
-		rate, err := strconv.ParseUint(split[1], 10, 64)
+		rate, err := strconv.ParseUint(val, 10, 64)
 		if err != nil {
-			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>. Number must be a positive integer", val)
+			return nil, fmt.Errorf("invalid rate for device: %s. The correct format is <device-path>:<number>. Number must be a positive integer", dev)
 		}
-		td[split[0]] = specs.LinuxThrottleDevice{Rate: rate}
+		td[key] = specs.LinuxThrottleDevice{Rate: rate}
 	}
 	return td, nil
 }
@@ -1089,42 +1124,42 @@ func parseSecrets(secrets []string) ([]specgen.Secret, map[string]string, error)
 		}
 
 		for _, val := range split {
-			kv := strings.SplitN(val, "=", 2)
-			if len(kv) < 2 {
+			name, value, hasValue := strings.Cut(val, "=")
+			if !hasValue {
 				return nil, nil, fmt.Errorf("option %s must be in form option=value: %w", val, secretParseError)
 			}
-			switch kv[0] {
+			switch name {
 			case "source":
-				source = kv[1]
+				source = value
 			case "type":
 				if secretType != "" {
 					return nil, nil, fmt.Errorf("cannot set more than one secret type: %w", secretParseError)
 				}
-				if kv[1] != "mount" && kv[1] != "env" {
-					return nil, nil, fmt.Errorf("type %s is invalid: %w", kv[1], secretParseError)
+				if value != "mount" && value != "env" {
+					return nil, nil, fmt.Errorf("type %s is invalid: %w", value, secretParseError)
 				}
-				secretType = kv[1]
+				secretType = value
 			case "target":
-				target = kv[1]
+				target = value
 			case "mode":
 				mountOnly = true
-				mode64, err := strconv.ParseUint(kv[1], 8, 32)
+				mode64, err := strconv.ParseUint(value, 8, 32)
 				if err != nil {
-					return nil, nil, fmt.Errorf("mode %s invalid: %w", kv[1], secretParseError)
+					return nil, nil, fmt.Errorf("mode %s invalid: %w", value, secretParseError)
 				}
 				mode = uint32(mode64)
 			case "uid", "UID":
 				mountOnly = true
-				uid64, err := strconv.ParseUint(kv[1], 10, 32)
+				uid64, err := strconv.ParseUint(value, 10, 32)
 				if err != nil {
-					return nil, nil, fmt.Errorf("UID %s invalid: %w", kv[1], secretParseError)
+					return nil, nil, fmt.Errorf("UID %s invalid: %w", value, secretParseError)
 				}
 				uid = uint32(uid64)
 			case "gid", "GID":
 				mountOnly = true
-				gid64, err := strconv.ParseUint(kv[1], 10, 32)
+				gid64, err := strconv.ParseUint(value, 10, 32)
 				if err != nil {
-					return nil, nil, fmt.Errorf("GID %s invalid: %w", kv[1], secretParseError)
+					return nil, nil, fmt.Errorf("GID %s invalid: %w", value, secretParseError)
 				}
 				gid = uint32(gid64)
 
@@ -1189,17 +1224,17 @@ func parseLinuxResourcesDeviceAccess(device string) (specs.LinuxDeviceCgroup, er
 		return specs.LinuxDeviceCgroup{}, fmt.Errorf("invalid device type in device-access-add: %s", devType)
 	}
 
-	number := strings.SplitN(value[1], ":", 2)
-	if number[0] != "*" {
-		i, err := strconv.ParseUint(number[0], 10, 64)
+	majorNumber, minorNumber, hasMinor := strings.Cut(value[1], ":")
+	if majorNumber != "*" {
+		i, err := strconv.ParseUint(majorNumber, 10, 64)
 		if err != nil {
 			return specs.LinuxDeviceCgroup{}, err
 		}
 		m := int64(i)
 		major = &m
 	}
-	if len(number) == 2 && number[1] != "*" {
-		i, err := strconv.ParseUint(number[1], 10, 64)
+	if hasMinor && minorNumber != "*" {
+		i, err := strconv.ParseUint(minorNumber, 10, 64)
 		if err != nil {
 			return specs.LinuxDeviceCgroup{}, err
 		}
@@ -1249,11 +1284,11 @@ func GetResources(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions)
 
 	unifieds := make(map[string]string)
 	for _, unified := range c.CgroupConf {
-		splitUnified := strings.SplitN(unified, "=", 2)
-		if len(splitUnified) < 2 {
+		key, val, hasVal := strings.Cut(unified, "=")
+		if !hasVal {
 			return nil, errors.New("--cgroup-conf must be formatted KEY=VALUE")
 		}
-		unifieds[splitUnified[0]] = splitUnified[1]
+		unifieds[key] = val
 	}
 	if len(unifieds) > 0 {
 		s.ResourceLimits.Unified = unifieds
@@ -1264,3 +1299,27 @@ func GetResources(s *specgen.SpecGenerator, c *entities.ContainerCreateOptions)
 	}
 	return s.ResourceLimits, nil
 }
+
+func UpdateMajorAndMinorNumbers(resources *specs.LinuxResources, devicesLimits *define.UpdateContainerDevicesLimits) (*specs.LinuxResources, error) {
+	spec := specgen.SpecGenerator{}
+	spec.ResourceLimits = &specs.LinuxResources{}
+	if resources != nil {
+		spec.ResourceLimits = resources
+	}
+
+	spec.WeightDevice = devicesLimits.GetMapOfLinuxWeightDevice()
+	spec.ThrottleReadBpsDevice = devicesLimits.GetMapOfDeviceReadBPs()
+	spec.ThrottleWriteBpsDevice = devicesLimits.GetMapOfDeviceWriteBPs()
+	spec.ThrottleReadIOPSDevice = devicesLimits.GetMapOfDeviceReadIOPs()
+	spec.ThrottleWriteIOPSDevice = devicesLimits.GetMapOfDeviceWriteIOPs()
+
+	err := specgen.WeightDevices(&spec)
+	if err != nil {
+		return nil, err
+	}
+	err = specgen.FinishThrottleDevices(&spec)
+	if err != nil {
+		return nil, err
+	}
+	return spec.ResourceLimits, nil
+}
