@@ -1,3 +1,5 @@
+//go:build linux || freebsd
+
 package integration
 
 import (
@@ -8,8 +10,8 @@ import (
 	"strings"
 	"time"
 
-	"github.com/containers/podman/v4/libpod/define"
-	. "github.com/containers/podman/v4/test/utils"
+	"github.com/containers/podman/v5/libpod/define"
+	. "github.com/containers/podman/v5/test/utils"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 	. "github.com/onsi/gomega/gbytes"
@@ -19,7 +21,9 @@ import (
 var _ = Describe("Verify podman containers.conf usage", func() {
 
 	BeforeEach(func() {
-		os.Setenv("CONTAINERS_CONF", "config/containers.conf")
+		confPath, err := filepath.Abs("config/containers.conf")
+		Expect(err).ToNot(HaveOccurred())
+		os.Setenv("CONTAINERS_CONF", confPath)
 		if IsRemote() {
 			podmanTest.RestartRemoteService()
 		}
@@ -263,6 +267,29 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 		Expect(out).To(ContainSubstring("alpine"))
 	})
 
+	It("using journald for container with container log_tag override", func() {
+		SkipIfJournaldUnavailable()
+		os.Setenv("CONTAINERS_CONF", "config/containers-journald.conf")
+		if IsRemote() {
+			podmanTest.RestartRemoteService()
+		}
+		logc := podmanTest.Podman([]string{"run", "-d", "--log-opt", "tag=OverriddenTag", ALPINE, "sh", "-c", "echo podman; sleep 0.1; echo podman; sleep 0.1; echo podman"})
+		logc.WaitWithDefaultTimeout()
+		Expect(logc).Should(ExitCleanly())
+		cid := logc.OutputToString()
+
+		wait := podmanTest.Podman([]string{"wait", cid})
+		wait.WaitWithDefaultTimeout()
+		Expect(wait).Should(ExitCleanly())
+
+		// Flake prevention: journalctl makes no timeliness guarantees.
+		time.Sleep(1 * time.Second)
+		cmd := exec.Command("journalctl", "--no-pager", "-o", "json", "--output-fields=CONTAINER_TAG", fmt.Sprintf("CONTAINER_ID_FULL=%s", cid))
+		out, err := cmd.CombinedOutput()
+		Expect(err).ToNot(HaveOccurred())
+		Expect(out).To(ContainSubstring("OverriddenTag"))
+	})
+
 	It("add volumes", func() {
 		conffile := filepath.Join(podmanTest.TempDir, "container.conf")
 
@@ -292,7 +319,7 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 		session = podmanTest.Podman([]string{"run", "--rm", "--net", "host", fedoraMinimal, "cat", "/proc/sys/net/ipv4/ping_group_range"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitCleanly())
-		Expect(session.OutputToString()).ToNot((ContainSubstring("1000")))
+		Expect(session.OutputToString()).ToNot(ContainSubstring("1000"))
 	})
 
 	It("search domain", func() {
@@ -344,10 +371,11 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 		Expect(session.OutputToString()).To(ContainSubstring("HST"))
 
 		// verify flag still overrides
-		session = podmanTest.Podman([]string{"run", "--tz", "EST", ALPINE, "date", "+'%H %Z'"})
+		// Arizona does not observe DST so this command is safe at all times of the year
+		session = podmanTest.Podman([]string{"run", "--tz", "America/Phoenix", ALPINE, "date", "+'%H %Z'"})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(ExitCleanly())
-		Expect(session.OutputToString()).To(ContainSubstring("EST"))
+		Expect(session.OutputToString()).To(ContainSubstring("MST"))
 	})
 
 	It("add umask", func() {
@@ -441,8 +469,7 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 	It("--add-host and no-hosts=true fails", func() {
 		session := podmanTest.Podman([]string{"run", "-dt", "--add-host", "test1:127.0.0.1", ALPINE, "top"})
 		session.WaitWithDefaultTimeout()
-		Expect(session).To(ExitWithError())
-		Expect(session.ErrorToString()).To(ContainSubstring("--no-hosts and --add-host cannot be set together"))
+		Expect(session).To(ExitWithError(125, "--no-hosts and --add-host cannot be set together"))
 
 		session = podmanTest.Podman([]string{"run", "-dt", "--add-host", "test1:127.0.0.1", "--no-hosts=false", ALPINE, "top"})
 		session.WaitWithDefaultTimeout()
@@ -461,6 +488,79 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 		Expect(session.OutputToString()).To(ContainSubstring("test"))
 	})
 
+	Describe("base_hosts_file in containers.conf", func() {
+		var baseHostsFile string
+		var session *PodmanSessionIntegration
+
+		JustBeforeEach(func() {
+			conffile := filepath.Join(podmanTest.TempDir, "containers.conf")
+			err = os.WriteFile(conffile, []byte(fmt.Sprintf("[containers]\nbase_hosts_file=\"%s\"\nno_hosts=false\n", baseHostsFile)), 0755)
+			Expect(err).ToNot(HaveOccurred())
+			os.Setenv("CONTAINERS_CONF_OVERRIDE", conffile)
+			if IsRemote() {
+				podmanTest.RestartRemoteService()
+			}
+
+			dockerfile := strings.Join([]string{
+				`FROM quay.io/libpod/alpine:latest`,
+				`RUN echo '56.78.12.34 image.example.com' > /etc/hosts`,
+			}, "\n")
+			podmanTest.BuildImage(dockerfile, "foobar.com/hosts_test:latest", "false", "--no-hosts")
+
+			session = podmanTest.Podman([]string{"run", "--name", "hosts_test", "--hostname", "hosts_test.dev", "--rm", "foobar.com/hosts_test:latest", "cat", "/etc/hosts"})
+			session.WaitWithDefaultTimeout()
+			Expect(session).Should(ExitCleanly())
+		})
+
+		Describe("base_hosts_file=path", func() {
+			BeforeEach(func() {
+				hostsPath := filepath.Join(podmanTest.TempDir, "hosts")
+				err := os.WriteFile(hostsPath, []byte("12.34.56.78 file.example.com"), 0755)
+				Expect(err).ToNot(HaveOccurred())
+				baseHostsFile = hostsPath
+			})
+
+			It("should use the hosts file from the file path", func() {
+				Expect(session.OutputToString()).ToNot(ContainSubstring("56.78.12.34 image.example.com"))
+				Expect(session.OutputToString()).To(ContainSubstring("12.34.56.78 file.example.com"))
+				Expect(session.OutputToString()).To(ContainSubstring("127.0.0.1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("::1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("host.containers.internal host.docker.internal"))
+				Expect(session.OutputToString()).To(ContainSubstring("hosts_test.dev hosts_test"))
+			})
+		})
+
+		Describe("base_hosts_file=image", func() {
+			BeforeEach(func() {
+				baseHostsFile = "image"
+			})
+
+			It("should use the hosts file from the container image", func() {
+				Expect(session.OutputToString()).To(ContainSubstring("56.78.12.34 image.example.com"))
+				Expect(session.OutputToString()).ToNot(ContainSubstring("12.34.56.78 file.example.com"))
+				Expect(session.OutputToString()).To(ContainSubstring("127.0.0.1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("::1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("host.containers.internal host.docker.internal"))
+				Expect(session.OutputToString()).To(ContainSubstring("hosts_test.dev hosts_test"))
+			})
+		})
+
+		Describe("base_hosts_file=none", func() {
+			BeforeEach(func() {
+				baseHostsFile = "none"
+			})
+
+			It("should not use any hosts files", func() {
+				Expect(session.OutputToString()).ToNot(ContainSubstring("56.78.12.34 image.example.com"))
+				Expect(session.OutputToString()).ToNot(ContainSubstring("12.34.56.78 file.example.com"))
+				Expect(session.OutputToString()).To(ContainSubstring("127.0.0.1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("::1 localhost"))
+				Expect(session.OutputToString()).To(ContainSubstring("host.containers.internal host.docker.internal"))
+				Expect(session.OutputToString()).To(ContainSubstring("hosts_test.dev hosts_test"))
+			})
+		})
+	})
+
 	It("seccomp profile path", func() {
 		configPath := filepath.Join(podmanTest.TempDir, "containers.conf")
 		os.Setenv("CONTAINERS_CONF", configPath)
@@ -533,8 +633,7 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 		if !IsRemote() {
 			session = podmanTest.Podman([]string{"info", "--format", "{{.Store.ImageCopyTmpDir}}"})
 			session.WaitWithDefaultTimeout()
-			Expect(session).Should(Exit(125))
-			Expect(session.ErrorToString()).To(ContainSubstring("invalid image_copy_tmp_dir value \"storage1\" (relative paths are not accepted)"))
+			Expect(session).Should(ExitWithError(125, `invalid image_copy_tmp_dir value "storage1" (relative paths are not accepted)`))
 
 			os.Setenv("TMPDIR", "/hoge")
 			session = podmanTest.Podman([]string{"info", "--format", "{{.Store.ImageCopyTmpDir}}"})
@@ -573,18 +672,15 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 
 		result := podmanTest.Podman([]string{"pod", "create", "--infra-image", infra2})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring(error2String))
+		Expect(result).Should(ExitWithError(125, error2String))
 
 		result = podmanTest.Podman([]string{"pod", "create"})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring(errorString))
+		Expect(result).Should(ExitWithError(125, errorString))
 
 		result = podmanTest.Podman([]string{"create", "--pod", "new:pod1", ALPINE})
 		result.WaitWithDefaultTimeout()
-		Expect(result).Should(Exit(125))
-		Expect(result.ErrorToString()).To(ContainSubstring(errorString))
+		Expect(result).Should(ExitWithError(125, errorString))
 	})
 
 	It("set .engine.remote=true", func() {
@@ -679,8 +775,7 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 			podman.WaitWithDefaultTimeout()
 
 			if mode == "invalid" {
-				Expect(podman).Should(Exit(125))
-				Expect(podman.ErrorToString()).Should(ContainSubstring("invalid default_rootless_network_cmd option \"invalid\""))
+				Expect(podman).Should(ExitWithError(125, `invalid default_rootless_network_cmd option "invalid"`))
 				continue
 			}
 			Expect(podman).Should(ExitCleanly())
@@ -691,4 +786,109 @@ var _ = Describe("Verify podman containers.conf usage", func() {
 			Expect(inspect.OutputToString()).Should(Equal(mode))
 		}
 	})
+
+	startContainer := func(params ...string) string {
+		args := []string{"create"}
+		for _, param := range params {
+			if param == "--name" {
+				args = append(args, "--replace")
+				break
+			}
+		}
+		args = append(args, params...)
+		args = append(args, ALPINE, "true")
+
+		result := podmanTest.Podman(args)
+		result.WaitWithDefaultTimeout()
+		Expect(result).Should(ExitCleanly())
+		containerID := result.OutputToString()
+
+		return containerID
+	}
+
+	getContainerConfig := func(containerID string, formatParam string) string {
+		inspect := podmanTest.Podman([]string{"inspect", "--format", formatParam, containerID})
+		inspect.WaitWithDefaultTimeout()
+		value := inspect.OutputToString()
+		return value
+	}
+
+	It("podman containers.conf container_name_as_hostname", func() {
+
+		// With default containers.conf
+
+		// Start container with no options
+		containerID := startContainer()
+		hostname := getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should be the first 12 characters of the containerID
+		Expect(hostname).To(Equal(containerID[:12]))
+
+		// Start container with name
+		containerID = startContainer("--name", "cname1")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should still be the first 12 characters of the containerID
+		Expect(hostname).To(Equal(containerID[:12]))
+
+		// Start container with just hostname
+		containerID = startContainer("--hostname", "cname1.dev")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should now be "cname1.dev"
+		Expect(hostname).To(Equal("cname1.dev"))
+
+		// Start container with name and hostname
+		containerID = startContainer("--name", "cname1", "--hostname", "cname1.dev")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should now be "cname1.dev"
+		Expect(hostname).To(Equal("cname1.dev"))
+
+		// Create containers.conf override with container_name_as_hostname=true
+		conffile := filepath.Join(podmanTest.TempDir, "container.conf")
+		err := os.WriteFile(conffile, []byte("[containers]\ncontainer_name_as_hostname=true\n"), 0755)
+		Expect(err).ToNot(HaveOccurred())
+		os.Setenv("CONTAINERS_CONF_OVERRIDE", conffile)
+		if IsRemote() {
+			podmanTest.RestartRemoteService()
+		}
+
+		// Start container with no options
+		containerID = startContainer()
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		name := getContainerConfig(containerID, "{{ .Name }}")
+		// Hostname should be the auto generated container name with '_' removed
+		Expect(hostname).To(Equal(strings.ReplaceAll(name, "_", "")))
+
+		// Start container with name
+		containerID = startContainer("--name", "cname1")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should be the container name
+		Expect(hostname).To(Equal("cname1"))
+
+		// Start container with name containing '_'
+		containerID = startContainer("--name", "cname1_2_3")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should be the set container name with all '_' removed
+		Expect(hostname).To(Equal("cname123"))
+
+		// Start container with just hostname
+		containerID = startContainer("--hostname", "cname1.dev")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should now be "cname1.dev"
+		Expect(hostname).To(Equal("cname1.dev"))
+
+		// Start container with name and hostname
+		containerID = startContainer("--name", "cname1", "--hostname", "cname1.dev")
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		// Hostname should still be "cname1.dev"
+		Expect(hostname).To(Equal("cname1.dev"))
+
+		// Start container with name = 260 characters
+		longHostname := "cnabcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890.abcdefghijklmnopqrstuvwxyz1234567890"
+		containerID = startContainer("--name", longHostname)
+		hostname = getContainerConfig(containerID, "{{ .Config.Hostname }}")
+		name = getContainerConfig(containerID, "{{ .Name }}")
+		// Double check that name actually got set correctly
+		Expect(name).To(Equal(longHostname))
+		// Hostname should be the container name truncated to 253 characters
+		Expect(hostname).To(Equal(name[:253]))
+	})
 })
