@@ -1,3 +1,5 @@
+//go:build linux || freebsd
+
 package integration
 
 import (
@@ -8,11 +10,10 @@ import (
 	"strconv"
 	"text/template"
 
-	"github.com/containers/podman/v4/pkg/domain/entities"
-	. "github.com/containers/podman/v4/test/utils"
-	. "github.com/onsi/ginkgo"
+	"github.com/containers/podman/v5/pkg/domain/entities"
+	. "github.com/containers/podman/v5/test/utils"
+	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gexec"
 )
 
 type endpoint struct {
@@ -25,11 +26,6 @@ func (e *endpoint) Address() string {
 }
 
 var _ = Describe("Podman search", func() {
-	var (
-		tempdir    string
-		err        error
-		podmanTest *PodmanTestIntegration
-	)
 
 	const regFileContents = `
 [registries.search]
@@ -47,34 +43,10 @@ registries = ['{{.Host}}:{{.Port}}']
 registries = []`
 	registryFileBadTmpl := template.Must(template.New("registryFileBad").Parse(badRegFileContents))
 
-	const regFileContents2 = `
-[registries.search]
-registries = ['{{.Host}}:{{.Port}}', '{{.Host}}:6000']
-
-[registries.insecure]
-registries = ['{{.Host}}:{{.Port}}']`
-	registryFileTwoTmpl := template.Must(template.New("registryFileTwo").Parse(regFileContents2))
-
-	BeforeEach(func() {
-		tempdir, err = CreateTempDirInTempDir()
-		if err != nil {
-			os.Exit(1)
-		}
-
-		podmanTest = PodmanTestCreate(tempdir)
-		podmanTest.Setup()
-	})
-
-	AfterEach(func() {
-		podmanTest.Cleanup()
-		f := CurrentGinkgoTestDescription()
-		processTestResult(f)
-	})
-
 	It("podman search", func() {
 		search := podmanTest.Podman([]string{"search", "alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically(">", 1))
 		Expect(search.OutputToString()).To(ContainSubstring("alpine"))
 	})
@@ -82,41 +54,42 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search single registry flag", func() {
 		search := podmanTest.Podman([]string{"search", "quay.io/skopeo/stable:latest"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).To(ContainSubstring("quay.io/skopeo/stable"))
 	})
 
 	It("podman search image with description", func() {
-		search := podmanTest.Podman([]string{"search", "quay.io/libpod/whalesay"})
+		search := podmanTest.Podman([]string{"search", "quay.io/podman/stable"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		output := string(search.Out.Contents())
 		Expect(output).To(MatchRegexp(`(?m)NAME\s+DESCRIPTION$`))
-		Expect(output).To(MatchRegexp(`(?m)quay.io/libpod/whalesay\s+Static image used for automated testing.+$`))
+		Expect(output).To(MatchRegexp(`(?m)quay.io/podman/stable\s+.*PODMAN logo`))
 	})
 
 	It("podman search image with --compatible", func() {
-		search := podmanTest.Podman([]string{"search", "--compatible", "quay.io/libpod/whalesay"})
+		search := podmanTest.Podman([]string{"search", "--compatible", "quay.io/podman/stable"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		output := string(search.Out.Contents())
 		Expect(output).To(MatchRegexp(`(?m)NAME\s+DESCRIPTION\s+STARS\s+OFFICIAL\s+AUTOMATED$`))
 	})
 
 	It("podman search format flag", func() {
-		search := podmanTest.Podman([]string{"search", "--format", "table {{.Index}} {{.Name}}", "alpine"})
+		search := podmanTest.Podman([]string{"search", "--format", "table {{.Index}} {{.Name}}", "testdigest_v2s2"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically(">", 1))
-		Expect(search.OutputToString()).To(ContainSubstring("docker.io/library/alpine"))
+		Expect(search.OutputToString()).To(ContainSubstring("quay.io/libpod/testdigest_v2s2"))
 	})
 
 	It("podman search format json", func() {
-		search := podmanTest.Podman([]string{"search", "--format", "json", "busybox"})
+		search := podmanTest.Podman([]string{"search", "--format", "json", "testdigest_v2s1"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).To(BeValidJSON())
-		Expect(search.OutputToString()).To(ContainSubstring("docker.io/library/busybox"))
+		Expect(search.OutputToString()).To(ContainSubstring("quay.io/libpod/testdigest_v2s1"))
+		Expect(search.OutputToString()).To(ContainSubstring("Test image used by buildah regression tests"))
 
 		// Test for https://github.com/containers/podman/issues/11894
 		contents := make([]entities.ImageSearchReport, 0)
@@ -131,7 +104,7 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search format json list tags", func() {
 		search := podmanTest.Podman([]string{"search", "--list-tags", "--format", "json", ALPINE})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).To(BeValidJSON())
 		Expect(search.OutputToString()).To(ContainSubstring("quay.io/libpod/alpine"))
 		Expect(search.OutputToString()).To(ContainSubstring("3.10.2"))
@@ -142,7 +115,7 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search no-trunc=false flag", func() {
 		search := podmanTest.Podman([]string{"search", "--no-trunc=false", "alpine", "--format={{.Description}}"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 
 		for _, line := range search.OutputToStringArray() {
 			if len(line) > 44 {
@@ -152,26 +125,26 @@ registries = ['{{.Host}}:{{.Port}}']`
 	})
 
 	It("podman search limit flag", func() {
-		search := podmanTest.Podman([]string{"search", "docker.io/alpine"})
+		search := podmanTest.Podman([]string{"search", "quay.io/alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically(">", 10))
 
-		search = podmanTest.Podman([]string{"search", "--limit", "3", "docker.io/alpine"})
+		search = podmanTest.Podman([]string{"search", "--limit", "3", "quay.io/alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToStringArray()).To(HaveLen(4))
 
-		search = podmanTest.Podman([]string{"search", "--limit", "30", "docker.io/alpine"})
+		search = podmanTest.Podman([]string{"search", "--limit", "30", "quay.io/alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToStringArray()).To(HaveLen(31))
 	})
 
 	It("podman search with filter stars", func() {
 		search := podmanTest.Podman([]string{"search", "--filter", "stars=10", "--format", "{{.Stars}}", "alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		output := search.OutputToStringArray()
 		for i := 0; i < len(output); i++ {
 			Expect(strconv.Atoi(output[i])).To(BeNumerically(">=", 10))
@@ -181,7 +154,7 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search with filter is-official", func() {
 		search := podmanTest.Podman([]string{"search", "--filter", "is-official", "--format", "{{.Official}}", "alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		output := search.OutputToStringArray()
 		for i := 0; i < len(output); i++ {
 			Expect(output[i]).To(Equal("[OK]"))
@@ -191,13 +164,20 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search with filter is-automated", func() {
 		search := podmanTest.Podman([]string{"search", "--filter", "is-automated=false", "--format", "{{.Automated}}", "alpine"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		output := search.OutputToStringArray()
 		for i := 0; i < len(output); i++ {
 			Expect(output[i]).To(Equal(""))
 		}
 	})
 
+	It("podman search format list tags with custom", func() {
+		search := podmanTest.Podman([]string{"search", "--list-tags", "--format", "{{.Name}}", "--limit", "1", ALPINE})
+		search.WaitWithDefaultTimeout()
+		Expect(search).Should(ExitCleanly())
+		Expect(search.OutputToString()).To(Equal("quay.io/libpod/alpine"))
+	})
+
 	It("podman search attempts HTTP if tls-verify flag is set false", func() {
 		if podmanTest.Host.Arch == "ppc64le" {
 			Skip("No registry image for ppc64le")
@@ -207,21 +187,20 @@ registries = ['{{.Host}}:{{.Port}}']`
 			"-p", fmt.Sprintf("%d:5000", port),
 			REGISTRY_IMAGE, "/entrypoint.sh", "/etc/docker/registry/config.yml"})
 		fakereg.WaitWithDefaultTimeout()
-		Expect(fakereg).Should(Exit(0))
+		Expect(fakereg).Should(ExitCleanly())
 
 		if !WaitContainerReady(podmanTest, "registry", "listening on", 20, 1) {
 			Fail("Cannot start docker registry on port %s", port)
 		}
-		ep := endpoint{Port: fmt.Sprintf("%d", port), Host: "localhost"}
+		ep := endpoint{Port: strconv.Itoa(port), Host: "localhost"}
 		search := podmanTest.Podman([]string{"search",
 			fmt.Sprintf("%s/fake/image:andtag", ep.Address()), "--tls-verify=false"})
 		search.WaitWithDefaultTimeout()
 
 		// if this test succeeded, there will be no output (there is no entry named fake/image:andtag in an empty registry)
 		// and the exit code will be 0
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).Should(BeEmpty())
-		Expect(search.ErrorToString()).Should(BeEmpty())
 	})
 
 	It("podman search in local registry", func() {
@@ -233,28 +212,28 @@ registries = ['{{.Host}}:{{.Port}}']`
 			"-p", fmt.Sprintf("%d:5000", port), REGISTRY_IMAGE,
 			"/entrypoint.sh", "/etc/docker/registry/config.yml"})
 		registry.WaitWithDefaultTimeout()
-		Expect(registry).Should(Exit(0))
+		Expect(registry).Should(ExitCleanly())
 
 		if !WaitContainerReady(podmanTest, "registry3", "listening on", 20, 1) {
 			Fail("Cannot start docker registry on port %s", port)
 		}
-		ep := endpoint{Port: fmt.Sprintf("%d", port), Host: "localhost"}
+		ep := endpoint{Port: strconv.Itoa(port), Host: "localhost"}
 		err = podmanTest.RestoreArtifact(ALPINE)
 		Expect(err).ToNot(HaveOccurred())
 		image := fmt.Sprintf("%s/my-alpine", ep.Address())
-		push := podmanTest.Podman([]string{"push", "--tls-verify=false", "--remove-signatures", ALPINE, image})
+		push := podmanTest.Podman([]string{"push", "-q", "--tls-verify=false", "--remove-signatures", ALPINE, image})
 		push.WaitWithDefaultTimeout()
-		Expect(push).Should(Exit(0))
+		Expect(push).Should(ExitCleanly())
 		search := podmanTest.Podman([]string{"search", image, "--tls-verify=false"})
 		search.WaitWithDefaultTimeout()
 
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).ShouldNot(BeEmpty())
 
 		// podman search v2 registry with empty query
 		searchEmpty := podmanTest.Podman([]string{"search", fmt.Sprintf("%s/", ep.Address()), "--tls-verify=false"})
 		searchEmpty.WaitWithDefaultTimeout()
-		Expect(searchEmpty).Should(Exit(0))
+		Expect(searchEmpty).Should(ExitCleanly())
 		Expect(searchEmpty.OutputToStringArray()).ToNot(BeEmpty())
 		Expect(search.OutputToString()).To(ContainSubstring("my-alpine"))
 	})
@@ -265,11 +244,11 @@ registries = ['{{.Host}}:{{.Port}}']`
 		}
 
 		port := GetPort()
-		ep := endpoint{Port: fmt.Sprintf("%d", port), Host: "localhost"}
+		ep := endpoint{Port: strconv.Itoa(port), Host: "localhost"}
 		registry := podmanTest.Podman([]string{"run", "-d", "-p", fmt.Sprintf("%d:5000", port),
 			"--name", "registry4", REGISTRY_IMAGE, "/entrypoint.sh", "/etc/docker/registry/config.yml"})
 		registry.WaitWithDefaultTimeout()
-		Expect(registry).Should(Exit(0))
+		Expect(registry).Should(ExitCleanly())
 
 		if !WaitContainerReady(podmanTest, "registry4", "listening on", 20, 1) {
 			Fail("unable to start registry on port %s", port)
@@ -278,9 +257,9 @@ registries = ['{{.Host}}:{{.Port}}']`
 		err = podmanTest.RestoreArtifact(ALPINE)
 		Expect(err).ToNot(HaveOccurred())
 		image := fmt.Sprintf("%s/my-alpine", ep.Address())
-		push := podmanTest.Podman([]string{"push", "--tls-verify=false", "--remove-signatures", ALPINE, image})
+		push := podmanTest.Podman([]string{"push", "-q", "--tls-verify=false", "--remove-signatures", ALPINE, image})
 		push.WaitWithDefaultTimeout()
-		Expect(push).Should(Exit(0))
+		Expect(push).Should(ExitCleanly())
 
 		// registries.conf set up
 		var buffer bytes.Buffer
@@ -297,9 +276,8 @@ registries = ['{{.Host}}:{{.Port}}']`
 		search := podmanTest.Podman([]string{"search", image})
 		search.WaitWithDefaultTimeout()
 
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToString()).To(ContainSubstring("my-alpine"))
-		Expect(search.ErrorToString()).Should(BeEmpty())
 
 		// cleanup
 		resetRegistriesConfigEnv()
@@ -310,11 +288,11 @@ registries = ['{{.Host}}:{{.Port}}']`
 			Skip("No registry image for ppc64le")
 		}
 		port := GetPort()
-		ep := endpoint{Port: fmt.Sprintf("%d", port), Host: "localhost"}
+		ep := endpoint{Port: strconv.Itoa(port), Host: "localhost"}
 		registry := podmanTest.Podman([]string{"run", "-d", "-p", fmt.Sprintf("%d:5000", port),
 			"--name", "registry5", REGISTRY_IMAGE})
 		registry.WaitWithDefaultTimeout()
-		Expect(registry).Should(Exit(0))
+		Expect(registry).Should(ExitCleanly())
 
 		if !WaitContainerReady(podmanTest, "registry5", "listening on", 20, 1) {
 			Fail("Cannot start docker registry on port %s", port)
@@ -323,9 +301,9 @@ registries = ['{{.Host}}:{{.Port}}']`
 		err = podmanTest.RestoreArtifact(ALPINE)
 		Expect(err).ToNot(HaveOccurred())
 		image := fmt.Sprintf("%s/my-alpine", ep.Address())
-		push := podmanTest.Podman([]string{"push", "--tls-verify=false", "--remove-signatures", ALPINE, image})
+		push := podmanTest.Podman([]string{"push", "-q", "--tls-verify=false", "--remove-signatures", ALPINE, image})
 		push.WaitWithDefaultTimeout()
-		Expect(push).Should(Exit(0))
+		Expect(push).Should(ExitCleanly())
 
 		var buffer bytes.Buffer
 		err = registryFileTmpl.Execute(&buffer, ep)
@@ -337,9 +315,8 @@ registries = ['{{.Host}}:{{.Port}}']`
 		search := podmanTest.Podman([]string{"search", image, "--tls-verify=true"})
 		search.WaitWithDefaultTimeout()
 
-		Expect(search).Should(Exit(125))
+		Expect(search).Should(ExitWithError(125, fmt.Sprintf(`couldn't search registry "localhost:%d": pinging container registry localhost:%d: Get "https://localhost:%d/v2/": http: server gave HTTP response to HTTPS client`, port, port, port)))
 		Expect(search.OutputToString()).Should(BeEmpty())
-		Expect(search.ErrorToString()).To(ContainSubstring("error"))
 
 		// cleanup
 		resetRegistriesConfigEnv()
@@ -350,11 +327,11 @@ registries = ['{{.Host}}:{{.Port}}']`
 			Skip("No registry image for ppc64le")
 		}
 		port := GetPort()
-		ep := endpoint{Port: fmt.Sprintf("%d", port), Host: "localhost"}
+		ep := endpoint{Port: strconv.Itoa(port), Host: "localhost"}
 		registry := podmanTest.Podman([]string{"run", "-d", "-p", fmt.Sprintf("%d:5000", port),
 			"--name", "registry6", REGISTRY_IMAGE})
 		registry.WaitWithDefaultTimeout()
-		Expect(registry).Should(Exit(0))
+		Expect(registry).Should(ExitCleanly())
 
 		if !WaitContainerReady(podmanTest, "registry6", "listening on", 20, 1) {
 			Fail("Cannot start docker registry on port %s", port)
@@ -363,9 +340,9 @@ registries = ['{{.Host}}:{{.Port}}']`
 		err = podmanTest.RestoreArtifact(ALPINE)
 		Expect(err).ToNot(HaveOccurred())
 		image := fmt.Sprintf("%s/my-alpine", ep.Address())
-		push := podmanTest.Podman([]string{"push", "--tls-verify=false", "--remove-signatures", ALPINE, image})
+		push := podmanTest.Podman([]string{"push", "-q", "--tls-verify=false", "--remove-signatures", ALPINE, image})
 		push.WaitWithDefaultTimeout()
-		Expect(push).Should(Exit(0))
+		Expect(push).Should(ExitCleanly())
 
 		var buffer bytes.Buffer
 		err = registryFileBadTmpl.Execute(&buffer, ep)
@@ -382,65 +359,8 @@ registries = ['{{.Host}}:{{.Port}}']`
 		search := podmanTest.Podman([]string{"search", image})
 		search.WaitWithDefaultTimeout()
 
-		Expect(search).Should(Exit(125))
+		Expect(search).Should(ExitWithError(125, fmt.Sprintf(`couldn't search registry "localhost:%d": pinging container registry localhost:%d: Get "https://localhost:%d/v2/": http: server gave HTTP response to HTTPS client`, port, port, port)))
 		Expect(search.OutputToString()).Should(BeEmpty())
-		Expect(search.ErrorToString()).To(ContainSubstring("error"))
-
-		// cleanup
-		resetRegistriesConfigEnv()
-	})
-
-	It("podman search doesn't attempt HTTP if one registry is not listed as insecure", func() {
-		if podmanTest.Host.Arch == "ppc64le" {
-			Skip("No registry image for ppc64le")
-		}
-		port1 := GetPort()
-		port2 := GetPort()
-		port3 := GetPort()
-		ep3 := endpoint{Port: fmt.Sprintf("%d", port3), Host: "localhost"}
-
-		registryLocal := podmanTest.Podman([]string{"run", "-d", "-p", fmt.Sprintf("%d", port1),
-			"--name", "registry7", REGISTRY_IMAGE})
-		registryLocal.WaitWithDefaultTimeout()
-		Expect(registryLocal).Should(Exit(0))
-
-		if !WaitContainerReady(podmanTest, "registry7", "listening on", 20, 1) {
-			Fail("Cannot start docker registry on port %s", port1)
-		}
-
-		registryLocal = podmanTest.Podman([]string{"run", "-d", "-p", fmt.Sprintf("%d:5000", port2), "--name", "registry8", REGISTRY_IMAGE})
-		registryLocal.WaitWithDefaultTimeout()
-		Expect(registryLocal).Should(Exit(0))
-
-		if !WaitContainerReady(podmanTest, "registry8", "listening on", 20, 1) {
-			Fail("Cannot start docker registry on port %s", port2)
-		}
-
-		err = podmanTest.RestoreArtifact(ALPINE)
-		Expect(err).ToNot(HaveOccurred())
-		push := podmanTest.Podman([]string{"push", "--tls-verify=false", "--remove-signatures", ALPINE, fmt.Sprintf("localhost:%d/my-alpine", port2)})
-		push.WaitWithDefaultTimeout()
-		Expect(push).Should(Exit(0))
-
-		// registries.conf set up
-		var buffer bytes.Buffer
-		err = registryFileTwoTmpl.Execute(&buffer, ep3)
-		Expect(err).ToNot(HaveOccurred())
-		podmanTest.setRegistriesConfigEnv(buffer.Bytes())
-		err = os.WriteFile(fmt.Sprintf("%s/registry8.conf", tempdir), buffer.Bytes(), 0644)
-		Expect(err).ToNot(HaveOccurred())
-
-		if IsRemote() {
-			podmanTest.RestartRemoteService()
-			defer podmanTest.RestartRemoteService()
-		}
-
-		search := podmanTest.Podman([]string{"search", "my-alpine"})
-		search.WaitWithDefaultTimeout()
-
-		Expect(search).Should(Exit(125))
-		Expect(search.OutputToString()).Should(BeEmpty())
-		Expect(search.ErrorToString()).To(ContainSubstring("error"))
 
 		// cleanup
 		resetRegistriesConfigEnv()
@@ -450,40 +370,49 @@ registries = ['{{.Host}}:{{.Port}}']`
 	It("podman search fail with nonexistent --authfile", func() {
 		search := podmanTest.Podman([]string{"search", "--authfile", "/tmp/nonexistent", ALPINE})
 		search.WaitWithDefaultTimeout()
-		Expect(search).To(ExitWithError())
+		Expect(search).To(ExitWithError(125, "credential file is not accessible: faccessat /tmp/nonexistent: no such file or directory"))
 	})
 
-	It("podman search with wildcards", func() {
+	// Registry is unreliable (#18484), this is another super-common flake
+	It("podman search with wildcards", FlakeAttempts(3), func() {
 		search := podmanTest.Podman([]string{"search", "registry.access.redhat.com/*openshift*"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically(">", 1))
 	})
 
 	It("podman search repository tags", func() {
-		search := podmanTest.Podman([]string{"search", "--list-tags", "--limit", "30", "docker.io/library/alpine"})
+		search := podmanTest.Podman([]string{"search", "--list-tags", "--limit", "30", "quay.io/podman/stable"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(search.OutputToStringArray()).To(HaveLen(31))
 
-		search = podmanTest.Podman([]string{"search", "--list-tags", "docker.io/library/alpine"})
+		search = podmanTest.Podman([]string{"search", "--list-tags", "quay.io/podman/stable"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically(">", 2))
 
-		search = podmanTest.Podman([]string{"search", "--filter=is-official", "--list-tags", "docker.io/library/alpine"})
+		search = podmanTest.Podman([]string{"search", "--filter=is-official", "--list-tags", "quay.io/podman/stable"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).To(ExitWithError())
+		Expect(search).To(ExitWithError(125, "filters are not applicable to list tags result"))
 
-		search = podmanTest.Podman([]string{"search", "--list-tags", "docker.io/library/"})
+		// With trailing slash
+		search = podmanTest.Podman([]string{"search", "--list-tags", "quay.io/podman/"})
 		search.WaitWithDefaultTimeout()
+		Expect(search).To(ExitWithError(125, `reference "podman/" must be a docker reference`))
+		Expect(search.OutputToStringArray()).To(BeEmpty())
+
+		// No trailing slash
+		search = podmanTest.Podman([]string{"search", "--list-tags", "quay.io/podman"})
+		search.WaitWithDefaultTimeout()
+		Expect(search).To(ExitWithError(125, "getting repository tags: fetching tags list: StatusCode: 404"))
 		Expect(search.OutputToStringArray()).To(BeEmpty())
 	})
 
 	It("podman search with limit over 100", func() {
 		search := podmanTest.Podman([]string{"search", "--limit", "100", "quay.io/podman"})
 		search.WaitWithDefaultTimeout()
-		Expect(search).Should(Exit(0))
+		Expect(search).Should(ExitCleanly())
 		Expect(len(search.OutputToStringArray())).To(BeNumerically("<=", 101))
 	})
 })
